@@ -2,24 +2,26 @@
 """
 Tests for numerical integration.
 """
+from __future__ import division, print_function, absolute_import
 
 import numpy
 from numpy import arange, zeros, array, dot, sqrt, cos, sin, eye, pi, exp, \
                   allclose
 
+from scipy.lib.six.moves import xrange
+
 from numpy.testing import assert_, TestCase, run_module_suite, \
         assert_array_almost_equal, assert_raises, assert_allclose, \
-        assert_array_equal
+        assert_array_equal, assert_equal
 from scipy.integrate import odeint, ode, complex_ode
 
 #------------------------------------------------------------------------------
 # Test ODE integrators
 #------------------------------------------------------------------------------
 
+
 class TestOdeint(TestCase):
-    """
-    Check integrate.odeint
-    """
+    # Check integrate.odeint
     def _do_problem(self, problem):
         t = arange(0.0, problem.stop_t, 0.05)
         z, infodict = odeint(problem.f, problem.z0, t, full_output=True)
@@ -28,13 +30,13 @@ class TestOdeint(TestCase):
     def test_odeint(self):
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.cmplx: continue
+            if problem.cmplx:
+                continue
             self._do_problem(problem)
 
+
 class TestOde(TestCase):
-    """
-    Check integrate.ode
-    """
+    # Check integrate.ode
     def _do_problem(self, problem, integrator, method='adams'):
 
         # ode has callback arguments in different order than odeint
@@ -55,42 +57,57 @@ class TestOde(TestCase):
         assert_(problem.verify(array([z]), problem.stop_t), (problem, method))
 
     def test_vode(self):
-        """Check the vode solver"""
+        # Check the vode solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.cmplx: continue
+            if problem.cmplx:
+                continue
             if not problem.stiff:
                 self._do_problem(problem, 'vode', 'adams')
             self._do_problem(problem, 'vode', 'bdf')
 
     def test_zvode(self):
-        """Check the zvode solver"""
+        # Check the zvode solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
             if not problem.stiff:
                 self._do_problem(problem, 'zvode', 'adams')
             self._do_problem(problem, 'zvode', 'bdf')
 
-    def test_dopri5(self):
-        """Check the dopri5 solver"""
+    def test_lsoda(self):
+        # Check the lsoda solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.cmplx: continue
-            if problem.stiff: continue
-            if hasattr(problem, 'jac'): continue
+            if problem.cmplx:
+                continue
+            self._do_problem(problem, 'lsoda')
+
+    def test_dopri5(self):
+        # Check the dopri5 solver
+        for problem_cls in PROBLEMS:
+            problem = problem_cls()
+            if problem.cmplx:
+                continue
+            if problem.stiff:
+                continue
+            if hasattr(problem, 'jac'):
+                continue
             self._do_problem(problem, 'dopri5')
 
     def test_dop853(self):
-        """Check the dop853 solver"""
+        # Check the dop853 solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.cmplx: continue
-            if problem.stiff: continue
-            if hasattr(problem, 'jac'): continue
+            if problem.cmplx:
+                continue
+            if problem.stiff:
+                continue
+            if hasattr(problem, 'jac'):
+                continue
             self._do_problem(problem, 'dop853')
 
     def test_concurrent_fail(self):
-        for sol in ('vode', 'zvode'):
+        for sol in ('vode', 'zvode', 'lsoda'):
             f = lambda t, y: 1.0
 
             r = ode(f).set_integrator(sol)
@@ -108,7 +125,7 @@ class TestOde(TestCase):
         f = lambda t, y: 1.0
 
         for k in xrange(3):
-            for sol in ('vode', 'zvode', 'dopri5', 'dop853'):
+            for sol in ('vode', 'zvode', 'lsoda', 'dopri5', 'dop853'):
                 r = ode(f).set_integrator(sol)
                 r.set_initial_value(0, 0)
 
@@ -138,10 +155,9 @@ class TestOde(TestCase):
                 assert_allclose(r.y, 0.3)
                 assert_allclose(r2.y, 0.2)
 
+
 class TestComplexOde(TestCase):
-    """
-    Check integrate.complex_ode
-    """
+    # Check integrate.complex_ode
     def _do_problem(self, problem, integrator, method='adams'):
 
         # ode has callback arguments in different order than odeint
@@ -163,7 +179,7 @@ class TestComplexOde(TestCase):
         assert_(problem.verify(array([z]), problem.stop_t), (problem, method))
 
     def test_vode(self):
-        """Check the vode solver"""
+        # Check the vode solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
             if not problem.stiff:
@@ -171,37 +187,161 @@ class TestComplexOde(TestCase):
             else:
                 self._do_problem(problem, 'vode', 'bdf')
 
-    def test_dopri5(self):
-        """Check the dopri5 solver"""
+    def test_lsoda(self):
+        # Check the lsoda solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.stiff: continue
-            if hasattr(problem, 'jac'): continue
+            self._do_problem(problem, 'lsoda')
+
+    def test_dopri5(self):
+        # Check the dopri5 solver
+        for problem_cls in PROBLEMS:
+            problem = problem_cls()
+            if problem.stiff:
+                continue
+            if hasattr(problem, 'jac'):
+                continue
             self._do_problem(problem, 'dopri5')
 
     def test_dop853(self):
-        """Check the dop853 solver"""
+        # Check the dop853 solver
         for problem_cls in PROBLEMS:
             problem = problem_cls()
-            if problem.stiff: continue
-            if hasattr(problem, 'jac'): continue
+            if problem.stiff:
+                continue
+            if hasattr(problem, 'jac'):
+                continue
             self._do_problem(problem, 'dop853')
 
+class TestSolout(TestCase):
+    # Check integrate.ode correctly handles solout for dopri5 and dop853
+    def _run_solout_test(self, integrator):
+        # Check correct usage of solout
+        ts = []
+        ys = []
+        t0 = 0.0
+        tend = 10.0
+        y0 = [1.0, 2.0]
+        def solout(t, y):
+            ts.append(t)
+            ys.append(y.copy())
+        def rhs(t, y):
+            return [y[0] + y[1], -y[1]**2]
+        ig = ode(rhs).set_integrator(integrator)
+        ig.set_solout(solout)
+        ig.set_initial_value(y0, t0)
+        ret = ig.integrate(tend)
+        assert_array_equal(ys[0], y0)
+        assert_array_equal(ys[-1], ret)
+        assert_equal(ts[0], t0)
+        assert_equal(ts[-1], tend)
+
+    def test_solout(self):
+        for integrator in ('dopri5', 'dop853'):
+            self._run_solout_test(integrator)
+
+    def _run_solout_break_test(self, integrator):
+        # Check correct usage of stopping via solout
+        ts = []
+        ys = []
+        t0 = 0.0
+        tend = 10.0
+        y0 = [1.0, 2.0]
+        def solout(t, y):
+            ts.append(t)
+            ys.append(y.copy())
+            if t > tend/2.0:
+                return -1
+        def rhs(t, y):
+            return [y[0] + y[1], -y[1]**2]
+        ig = ode(rhs).set_integrator(integrator)
+        ig.set_solout(solout)
+        ig.set_initial_value(y0, t0)
+        ret = ig.integrate(tend)
+        assert_array_equal(ys[0], y0)
+        assert_array_equal(ys[-1], ret)
+        assert_equal(ts[0], t0)
+        assert_(ts[-1] > tend/2.0)
+        assert_(ts[-1] < tend)
+
+    def test_solout_break(self):
+        for integrator in ('dopri5', 'dop853'):
+            self._run_solout_break_test(integrator)
+
+
+class TestComplexSolout(TestCase):
+    # Check integrate.ode correctly handles solout for dopri5 and dop853
+    def _run_solout_test(self, integrator):
+        # Check correct usage of solout
+        ts = []
+        ys = []
+        t0 = 0.0
+        tend = 20.0
+        y0 = [0.0]
+        def solout(t, y):
+            ts.append(t)
+            ys.append(y.copy())
+        def rhs(t, y):
+            return [1.0/(t - 10.0 - 1j)]
+        ig = complex_ode(rhs).set_integrator(integrator)
+        ig.set_solout(solout)
+        ig.set_initial_value(y0, t0)
+        ret = ig.integrate(tend)
+        assert_array_equal(ys[0], y0)
+        assert_array_equal(ys[-1], ret)
+        assert_equal(ts[0], t0)
+        assert_equal(ts[-1], tend)
+
+    def test_solout(self):
+        for integrator in ('dopri5', 'dop853'):
+            self._run_solout_test(integrator)
+
+    def _run_solout_break_test(self, integrator):
+        # Check correct usage of stopping via solout
+        ts = []
+        ys = []
+        t0 = 0.0
+        tend = 20.0
+        y0 = [0.0]
+        def solout(t, y):
+            ts.append(t)
+            ys.append(y.copy())
+            if t > tend/2.0:
+                return -1
+        def rhs(t, y):
+            return [1.0/(t - 10.0 - 1j)]
+        ig = complex_ode(rhs).set_integrator(integrator)
+        ig.set_solout(solout)
+        ig.set_initial_value(y0, t0)
+        ret = ig.integrate(tend)
+        assert_array_equal(ys[0], y0)
+        assert_array_equal(ys[-1], ret)
+        assert_equal(ts[0], t0)
+        assert_(ts[-1] > tend/2.0)
+        assert_(ts[-1] < tend)
+
+    def test_solout_break(self):
+        for integrator in ('dopri5', 'dop853'):
+            self._run_solout_break_test(integrator)
+
+
 #------------------------------------------------------------------------------
 # Test problems
 #------------------------------------------------------------------------------
 
+
 class ODE:
     """
     ODE problem
     """
-    stiff   = False
-    cmplx   = False
-    stop_t  = 1
-    z0      = []
+    stiff = False
+    cmplx = False
+    stop_t = 1
+    z0 = []
+
+    atol = 1e-6
+    rtol = 1e-5
 
-    atol    = 1e-6
-    rtol    = 1e-5
 
 class SimpleOscillator(ODE):
     r"""
@@ -210,8 +350,8 @@ class SimpleOscillator(ODE):
     Solution::
         u(t) = u_0*cos(sqrt(k/m)*t)+\dot{u}_0*sin(sqrt(k/m)*t)/sqrt(k/m)
     """
-    stop_t  = 1 + 0.09
-    z0      = array([1.0, 0.1], float)
+    stop_t = 1 + 0.09
+    z0 = array([1.0, 0.1], float)
 
     k = 4.0
     m = 1.0
@@ -227,11 +367,12 @@ class SimpleOscillator(ODE):
         u = self.z0[0]*cos(omega*t)+self.z0[1]*sin(omega*t)/omega
         return allclose(u, zs[:,0], atol=self.atol, rtol=self.rtol)
 
+
 class ComplexExp(ODE):
     r"""The equation :lm:`\dot u = i u`"""
-    stop_t  = 1.23*pi
-    z0      = exp([1j,2j,3j,4j,5j])
-    cmplx   = True
+    stop_t = 1.23*pi
+    z0 = exp([1j,2j,3j,4j,5j])
+    cmplx = True
 
     def f(self, z, t):
         return 1j*z
@@ -243,14 +384,16 @@ class ComplexExp(ODE):
         u = self.z0 * exp(1j*t)
         return allclose(u, zs, atol=self.atol, rtol=self.rtol)
 
+
 class Pi(ODE):
     r"""Integrate 1/(t + 1j) from t=-10 to t=10"""
-    stop_t  = 20
-    z0      = [0]
-    cmplx   = True
+    stop_t = 20
+    z0 = [0]
+    cmplx = True
 
     def f(self, z, t):
         return array([1./(t - 10 + 1j)])
+
     def verify(self, zs, t):
         u = -2j*numpy.arctan(10)
         return allclose(u, zs[-1,:], atol=self.atol, rtol=self.rtol)
@@ -259,39 +402,47 @@ PROBLEMS = [SimpleOscillator, ComplexExp, Pi]
 
 #------------------------------------------------------------------------------
 
+
 def f(t, x):
     dxdt = [x[1], -x[0]]
     return dxdt
 
+
 def jac(t, x):
-    j = array([[ 0.0, 1.0],
+    j = array([[0.0, 1.0],
                [-1.0, 0.0]])
     return j
 
+
 def f1(t, x, omega):
     dxdt = [omega*x[1], -omega*x[0]]
     return dxdt
 
+
 def jac1(t, x, omega):
-    j = array([[ 0.0, omega],
+    j = array([[0.0, omega],
                [-omega, 0.0]])
     return j
 
+
 def f2(t, x, omega1, omega2):
     dxdt = [omega1*x[1], -omega2*x[0]]
     return dxdt
 
+
 def jac2(t, x, omega1, omega2):
-    j = array([[ 0.0, omega1],
+    j = array([[0.0, omega1],
                [-omega2, 0.0]])
     return j
 
+
 def fv(t, x, omega):
     dxdt = [omega[0]*x[1], -omega[1]*x[0]]
     return dxdt
 
+
 def jacv(t, x, omega):
-    j = array([[ 0.0, omega[0]],
+    j = array([[0.0, omega[0]],
                [-omega[1], 0.0]])
     return j
 
@@ -374,5 +525,10 @@ class ZVODECheckParameterUse(ODECheckParameterUse, TestCase):
     solver_uses_jac = True
 
 
+class LSODACheckParameterUse(ODECheckParameterUse, TestCase):
+    solver_name = 'lsoda'
+    solver_uses_jac = True
+
+
 if __name__ == "__main__":
     run_module_suite()
