@@ -1,79 +1,6 @@
 """
 Python wrappers for Orthogonal Distance Regression (ODRPACK).
 
-Classes
-=======
-
-Data -- stores the data and weights to fit against
-
-RealData -- stores data with standard deviations and covariance matrices
-
-Model -- stores the model and its related information
-
-Output -- stores all of the output from an ODR run
-
-ODR -- collects all data and runs the fitting routine
-
-
-Exceptions
-==========
-`odr_error` :
-    error sometimes raised inside odr() and can be raised in the
-    fitting functions to tell ODRPACK to halt the procedure
-`odr_stop` :
-    error to raise in fitting functions to tell ODRPACK that the data or
-    parameters given are invalid
-
-Use
-===
-Basic use:
-
-1) Define the function you want to fit against.
-::
-
-  def f(B, x):
-      ''' Linear function y = m*x + b '''
-      return B[0]*x + B[1]
-
-      # B is a vector of the parameters.
-      # x is an array of the current x values.
-      # x is same format as the x passed to Data or RealData.
-
-      # Return an array in the same format as y passed to Data or RealData.
-
-2) Create a Model.
-::
-
-  linear = Model(f)
-
-3) Create a Data or RealData instance.
-::
-
-  mydata = Data(x, y, wd=1./power(sx,2), we=1./power(sy,2))
-
-or
-
-::
-
-  mydata = RealData(x, y, sx=sx, sy=sy)
-
-4) Instantiate ODR with your data, model and initial parameter estimate.
-::
-
-  myodr = ODR(mydata, linear, beta0=[1., 2.])
-
-5) Run the fit.
-::
-
-  myoutput = myodr.run()
-
-6) Examine output.
-::
-
-  myoutput.pprint()
-
-Read the docstrings and the accompanying tests for more advanced usage.
-
 Notes
 =====
 
@@ -109,16 +36,36 @@ robert.kern@gmail.com
 
 """
 
+from __future__ import division, print_function, absolute_import
+
 import numpy
 from scipy.odr import __odrpack
 
 __all__ = ['odr', 'odr_error', 'odr_stop', 'Data', 'RealData', 'Model',
            'Output', 'ODR']
 
-
 odr = __odrpack.odr
-odr_error = __odrpack.odr_error
-odr_stop = __odrpack.odr_stop
+
+
+class odr_error(Exception):
+    """
+    Exception indicating an error in fitting.
+
+    This is raised by `scipy.odr` if an error occurs during fitting.
+    """
+    pass
+
+
+class odr_stop(Exception):
+    """
+    Exception stopping fitting.
+
+    You can raise this exception in your objective function to tell
+    `scipy.odr` to stop fitting.
+    """
+    pass
+
+__odrpack._set_exceptions(odr_error, odr_stop)
 
 
 def _conv(obj, dtype=None):
@@ -162,10 +109,10 @@ def _report_error(info):
     if info >= 5:
         # questionable results or fatal error
 
-        I = (info/10000 % 10,
-             info/1000 % 10,
-             info/100 % 10,
-             info/10 % 10,
+        I = (info//10000 % 10,
+             info//1000 % 10,
+             info//100 % 10,
+             info//10 % 10,
              info % 10)
         problems = []
 
@@ -219,9 +166,7 @@ def _report_error(info):
 
 class Data(object):
     """
-    scipy.odr.Data(x, y=None, we=None, wd=None, fix=None, meta={})
-
-    The Data class stores the data to fit.
+    The data to fit.
 
     Parameters
     ----------
@@ -272,7 +217,7 @@ class Data(object):
         dimensions for all observations. A value of 0 fixes the observation,
         a value > 0 makes it free.
     meta : dict, optional
-        Freeform dictionary for metadata.
+        Free-form dictionary for metadata.
 
     Notes
     -----
@@ -303,24 +248,21 @@ class Data(object):
         self.fix = _conv(fix)
         self.meta = meta
 
-
     def set_meta(self, **kwds):
         """ Update the metadata dictionary with the keywords and data provided
         by keywords.
 
         Examples
         --------
-        data.set_meta(lab="Ph 7; Lab 26", title="Ag110 + Ag108 Decay")
+        >>> data.set_meta(lab="Ph 7; Lab 26", title="Ag110 + Ag108 Decay")
         """
 
         self.meta.update(kwds)
 
-
     def __getattr__(self, attr):
-        """ Dispatch aatribute access to the metadata dictionary.
+        """ Dispatch attribute access to the metadata dictionary.
         """
-
-        if attr in self.meta.keys():
+        if attr in self.meta:
             return self.meta[attr]
         else:
             raise AttributeError("'%s' not in metadata" % attr)
@@ -328,8 +270,8 @@ class Data(object):
 
 class RealData(Data):
     """
-    The RealData class stores the weightings as actual standard deviations
-    and/or covariances.
+    The data, with weightings as actual standard deviations and/or
+    covariances.
 
     Parameters
     ----------
@@ -355,20 +297,19 @@ class RealData(Data):
         `covy` is an array of covariance matrices and are converted to
         weights by performing a matrix inversion on each observation's
         covariance matrix.
-    fix : array_like
+    fix : array_like, optional
         The argument and member fix is the same as Data.fix and ODR.ifixx:
         It is an array of integers with the same shape as `x` that
         determines which input observations are treated as fixed. One can
         use a sequence of length m (the dimensionality of the input
         observations) to fix some dimensions for all observations. A value
         of 0 fixes the observation, a value > 0 makes it free.
-    meta : dict
-        Meta
+    meta : dict, optional
+        Free-form dictionary for metadata.
 
     Notes
     -----
-    The weights needed for ODRPACK are generated on-the-fly with
-    ``__getattr__`` trickery.
+    The weights `wd` and `we` are computed from provided values as follows:
 
     `sx` and `sy` are converted to weights by dividing 1.0 by their squares.
     For example, ``wd = 1./numpy.power(`sx`, 2)``.
@@ -413,7 +354,6 @@ class RealData(Data):
         self.fix = _conv(fix)
         self.meta = meta
 
-
     def _sd2wt(self, sd):
         """ Convert standard deviation to weights.
         """
@@ -436,16 +376,14 @@ class RealData(Data):
 
             return weights
 
-
     def __getattr__(self, attr):
-        lookup_tbl = {('wd', 'sx'):  (self._sd2wt, self.sx),
+        lookup_tbl = {('wd', 'sx'): (self._sd2wt, self.sx),
                       ('wd', 'covx'): (self._cov2wt, self.covx),
-                      ('we', 'sy'):  (self._sd2wt, self.sy),
+                      ('we', 'sy'): (self._sd2wt, self.sy),
                       ('we', 'covy'): (self._cov2wt, self.covy)}
 
-
         if attr not in ('wd', 'we'):
-            if attr in self.meta.keys():
+            if attr in self.meta:
                 return self.meta[attr]
             else:
                 raise AttributeError("'%s' not in metadata" % attr)
@@ -453,7 +391,7 @@ class RealData(Data):
             func, arg = lookup_tbl[(attr, self._ga_flags[attr])]
 
             if arg is not None:
-                return apply(func, (arg,))
+                return func(*(arg,))
             else:
                 return None
 
@@ -501,34 +439,36 @@ class Model(object):
     return a NumPy array. The `estimate` object takes an instance of the
     Data class.
 
-    Here are the rules for the shapes of the argument and return arrays :
+    Here are the rules for the shapes of the argument and return
+    arrays of the callback functions:
 
-      x -- if the input data is single-dimensional, then x is rank-1
-        array; i.e. x = array([1, 2, 3, ...]); x.shape = (n,)
-        If the input data is multi-dimensional, then x is a rank-2 array;
-        i.e., x = array([[1, 2, ...], [2, 4, ...]]); x.shape = (m, n) In
-        all cases, it has the same shape as the input data array passed to
-        odr(). m is the dimensionality of the input data, n is the number
+    `x`
+        if the input data is single-dimensional, then `x` is rank-1
+        array; i.e. ``x = array([1, 2, 3, ...]); x.shape = (n,)``
+        If the input data is multi-dimensional, then `x` is a rank-2 array;
+        i.e., ``x = array([[1, 2, ...], [2, 4, ...]]); x.shape = (m, n)``.
+        In all cases, it has the same shape as the input data array passed to
+        `odr`. `m` is the dimensionality of the input data, `n` is the number
         of observations.
-
-      y -- if the response variable is single-dimensional, then y is a
-        rank-1 array, i.e., y = array([2, 4, ...]); y.shape = (n,)
-        If the response variable is multi-dimensional, then y is a rank-2
-        array, i.e.,  y = array([[2, 4, ...], [3, 6, ...]]); y.shape =
-        (q, n) where q is the dimensionality of the response variable.
-
-      beta -- rank-1 array of length p where p is the number of parameters;
-        i.e. beta = array([B_1, B_2, ..., B_p])
-
-      fjacb -- if the response variable is multi-dimensional, then the
-        return array's shape is (q, p, n) such that fjacb(x,beta)[l,k,i] =
-        @f_l(X,B)/@B_k evaluated at the i'th data point.  If q == 1, then
-        the return array is only rank-2 and with shape (p, n).
-
-      fjacd -- as with fjacb, only the return array's shape is (q, m, n)
-        such that fjacd(x,beta)[l,j,i] = @f_l(X,B)/@X_j at the i'th data
-        point.  If q == 1, then the return array's shape is (m, n). If
-        m == 1, the shape is (q, n). If m == q == 1, the shape is (n,).
+    `y`
+        if the response variable is single-dimensional, then `y` is a
+        rank-1 array, i.e., ``y = array([2, 4, ...]); y.shape = (n,)``.
+        If the response variable is multi-dimensional, then `y` is a rank-2
+        array, i.e., ``y = array([[2, 4, ...], [3, 6, ...]]); y.shape =
+        (q, n)`` where `q` is the dimensionality of the response variable.
+    `beta`
+        rank-1 array of length `p` where `p` is the number of parameters;
+        i.e. ``beta = array([B_1, B_2, ..., B_p])``
+    `fjacb`
+        if the response variable is multi-dimensional, then the
+        return array's shape is `(q, p, n)` such that ``fjacb(x,beta)[l,k,i] =
+        d f_l(X,B)/d B_k`` evaluated at the i'th data point.  If `q == 1`, then
+        the return array is only rank-2 and with shape `(p, n)`.
+    `fjacd`
+        as with fjacb, only the return array's shape is `(q, m, n)`
+        such that ``fjacd(x,beta)[l,j,i] = d f_l(X,B)/d X_j`` at the i'th data
+        point.  If `q == 1`, then the return array's shape is `(m, n)`. If
+        `m == 1`, the shape is (q, n). If `m == q == 1`, the shape is `(n,)`.
 
     """
 
@@ -547,7 +487,6 @@ class Model(object):
         self.implicit = implicit
         self.meta = meta
 
-
     def set_meta(self, **kwds):
         """ Update the metadata dictionary with the keywords and data provided
         here.
@@ -559,12 +498,11 @@ class Model(object):
 
         self.meta.update(kwds)
 
-
     def __getattr__(self, attr):
         """ Dispatch attribute access to the metadata.
         """
 
-        if attr in self.meta.keys():
+        if attr in self.meta:
             return self.meta[attr]
         else:
             raise AttributeError("'%s' not in metadata" % attr)
@@ -574,9 +512,6 @@ class Output(object):
     """
     The Output class stores the output of an ODR run.
 
-    Takes one argument for initialization, the return value from the
-    function `odr`.
-
     Attributes
     ----------
     beta : ndarray
@@ -616,8 +551,9 @@ class Output(object):
 
     Notes
     -----
-    The attributes listed as "optional" above are only present if `odr` was run
-    with ``full_output=1``.
+    Takes one argument for initialization, the return value from the
+    function `odr`. The attributes listed as "optional" above are only
+    present if `odr` was run with ``full_output=1``.
 
     """
 
@@ -631,20 +567,19 @@ class Output(object):
             self.__dict__.update(output[3])
             self.stopreason = _report_error(self.info)
 
-
     def pprint(self):
         """ Pretty-print important results.
         """
 
-        print 'Beta:', self.beta
-        print 'Beta Std Error:', self.sd_beta
-        print 'Beta Covariance:', self.cov_beta
+        print('Beta:', self.beta)
+        print('Beta Std Error:', self.sd_beta)
+        print('Beta Covariance:', self.cov_beta)
         if hasattr(self, 'info'):
-            print 'Residual Variance:',self.res_var
-            print 'Inverse Condition #:', self.inv_condnum
-            print 'Reason(s) for Halting:'
+            print('Residual Variance:',self.res_var)
+            print('Inverse Condition #:', self.inv_condnum)
+            print('Reason(s) for Halting:')
             for r in self.stopreason:
-                print '  %s' % r
+                print('  %s' % r)
 
 
 class ODR(object):
@@ -661,6 +596,9 @@ class ODR(object):
         instance of the Data class
     model : Model class instance
         instance of the Model class
+
+    Other Parameters
+    ----------------
     beta0 : array_like of rank-1
         a rank-1 sequence of initial parameter values. Optional if
         model provides an "estimate" function to estimate these values.
@@ -706,7 +644,7 @@ class ODR(object):
     partol : float, optional
         float specifying the tolerance for convergence based on the relative
         change in the estimated parameters. The default value is eps**(2/3) for
-        explicit models and eps**(1/3) for implicit models. partol must be less
+        explicit models and ``eps**(1/3)`` for implicit models. partol must be less
         than 1.
     maxit : int, optional
         integer specifying the maximum number of iterations to perform. For
@@ -714,16 +652,16 @@ class ODR(object):
         defaults to 50.  For restarts, maxit is the number of additional
         iterations to perform and defaults to 10.
     stpb : array_like, optional
-        sequence (len(stpb) == len(beta0)) of relative step sizes to compute
+        sequence (``len(stpb) == len(beta0)``) of relative step sizes to compute
         finite difference derivatives wrt the parameters.
     stpd : optional
-        array (stpd.shape == data.x.shape or stpd.shape == (m,)) of relative
+        array (``stpd.shape == data.x.shape`` or ``stpd.shape == (m,)``) of relative
         step sizes to compute finite difference derivatives wrt the input
         variable errors. If stpd is a rank-1 array with length m (the
         dimensionality of the input variable), then the values are broadcast to
         all observations.
     sclb : array_like, optional
-        sequence (len(stpb) == len(beta0)) of scaling factors for the
+        sequence (``len(stpb) == len(beta0)``) of scaling factors for the
         parameters.  The purpose of these scaling factors are to scale all of
         the parameters to around unity. Normally appropriate scaling factors
         are computed if this argument is not specified. Specify them yourself
@@ -739,8 +677,15 @@ class ODR(object):
     iwork : ndarray, optional
         array to hold the integer-valued working data for ODRPACK. When
         restarting, takes the value of self.output.iwork.
-    output : Output class instance
-        an instance if the Output class containing all of the returned
+
+    Attributes
+    ----------
+    data : Data
+        The data for this fit
+    model : Model
+        The model used in fit
+    output : Output
+        An instance if the Output class containing all of the returned
         data from an invocation of ODR.run() or ODR.restart()
 
     """
@@ -851,8 +796,8 @@ class ODR(object):
         res = self.model.fcn(*arglist)
 
         if res.shape not in fcn_perms:
-            print res.shape
-            print fcn_perms
+            print(res.shape)
+            print(fcn_perms)
             raise odr_error("fcn does not output %s-shaped array" % y_s)
 
         if self.model.fjacd is not None:
@@ -917,7 +862,6 @@ class ODR(object):
         else:
             self.work = numpy.zeros((lwork,), float)
 
-
     def set_job(self, fit_type=None, deriv=None, var_calc=None,
         del_init=None, restart=None):
         """
@@ -977,10 +921,10 @@ class ODR(object):
         if self.job is None:
             job_l = [0, 0, 0, 0, 0]
         else:
-            job_l = [self.job / 10000 % 10,
-                     self.job / 1000 % 10,
-                     self.job / 100 % 10,
-                     self.job / 10 % 10,
+            job_l = [self.job // 10000 % 10,
+                     self.job // 1000 % 10,
+                     self.job // 100 % 10,
+                     self.job // 10 % 10,
                      self.job % 10]
 
         if fit_type in (0, 1, 2):
@@ -997,7 +941,6 @@ class ODR(object):
         self.job = (job_l[0]*10000 + job_l[1]*1000 +
                     job_l[2]*100 + job_l[3]*10 + job_l[4])
 
-
     def set_iprint(self, init=None, so_init=None,
         iter=None, so_iter=None, iter_step=None, final=None, so_final=None):
         """ Set the iprint parameter for the printing of computation reports.
@@ -1027,20 +970,20 @@ class ODR(object):
         if self.iprint is None:
             self.iprint = 0
 
-        ip = [self.iprint / 1000 % 10,
-              self.iprint / 100 % 10,
-              self.iprint / 10 % 10,
+        ip = [self.iprint // 1000 % 10,
+              self.iprint // 100 % 10,
+              self.iprint // 10 % 10,
               self.iprint % 10]
 
         # make a list to convert iprint digits to/from argument inputs
         #                   rptfile, stdout
-        ip2arg = [[0, 0], # none,  none
-                  [1, 0], # short, none
-                  [2, 0], # long,  none
-                  [1, 1], # short, short
-                  [2, 1], # long,  short
-                  [1, 2], # short, long
-                  [2, 2]] # long,  long
+        ip2arg = [[0, 0],  # none,  none
+                  [1, 0],  # short, none
+                  [2, 0],  # long,  none
+                  [1, 1],  # short, short
+                  [2, 1],  # long,  short
+                  [1, 2],  # short, long
+                  [2, 2]]  # long,  long
 
         if (self.rptfile is None and
             (so_init is not None or
@@ -1074,7 +1017,6 @@ class ODR(object):
 
         self.iprint = ip[0]*1000 + ip[1]*100 + ip[2]*10 + ip[3]
 
-
     def run(self):
         """ Run the fitting routine with all of the information given.
 
@@ -1090,7 +1032,7 @@ class ODR(object):
                  'ndigit', 'taufac', 'sstol', 'partol', 'maxit', 'stpb',
                  'stpd', 'sclb', 'scld', 'work', 'iwork']
 
-        if self.delta0 is not None and self.job % 1000 / 10 == 1:
+        if self.delta0 is not None and self.job % 1000 // 10 == 1:
             # delta0 provided and fit is not a restart
             self._gen_work()
 
@@ -1116,11 +1058,10 @@ class ODR(object):
             if obj is not None:
                 kwds[attr] = obj
 
-        self.output = Output(apply(odr, args, kwds))
+        self.output = Output(odr(*args, **kwds))
 
         return self.output
 
-
     def restart(self, iter=None):
         """ Restarts the run with iter more iterations.
 
