@@ -1,18 +1,28 @@
 """Base class for sparse matrices"""
+from __future__ import division, print_function, absolute_import
 
 __all__ = ['spmatrix', 'isspmatrix', 'issparse',
         'SparseWarning','SparseEfficiencyWarning']
 
+import sys
 from warnings import warn
 
 import numpy as np
 
-from sputils import isdense, isscalarlike, isintlike
+from scipy.lib.six.moves import xrange
+from .sputils import isdense, isscalarlike, isintlike
 
 
-class SparseWarning(Warning): pass
-class SparseFormatWarning(SparseWarning): pass
-class SparseEfficiencyWarning(SparseWarning): pass
+class SparseWarning(Warning):
+    pass
+
+
+class SparseFormatWarning(SparseWarning):
+    pass
+
+
+class SparseEfficiencyWarning(SparseWarning):
+    pass
 
 
 # The formats that we might potentially understand.
@@ -41,6 +51,7 @@ _formats = {'csc':[0, "Compressed Sparse Column"],
 
 MAXPRINT = 50
 
+
 class spmatrix(object):
     """ This class provides a base class for all sparse matrices.  It
     cannot be instantiated.  Most of the work is provided by subclasses.
@@ -48,6 +59,7 @@ class spmatrix(object):
 
     __array_priority__ = 10.1
     ndim = 2
+
     def __init__(self, maxprint=MAXPRINT):
         self.format = self.__class__.__name__[:3]
         self._shape = None
@@ -63,11 +75,11 @@ class spmatrix(object):
             raise ValueError("Only two-dimensional sparse arrays "
                                      "are supported.")
         try:
-            shape = int(shape[0]),int(shape[1]) #floats, other weirdness
+            shape = int(shape[0]),int(shape[1])  # floats, other weirdness
         except:
             raise TypeError('invalid shape')
 
-        if not (shape[0] >= 1 and shape[1] >= 1):
+        if not (shape[0] >= 0 and shape[1] >= 0):
             raise ValueError('invalid shape')
 
         if (self._shape != shape) and (self._shape is not None):
@@ -115,7 +127,7 @@ class spmatrix(object):
             maxprint = MAXPRINT
         return maxprint
 
-    #def typecode(self):
+    # def typecode(self):
     #    try:
     #        typ = self.dtype.char
     #    except AttributeError:
@@ -145,27 +157,32 @@ class spmatrix(object):
     def __str__(self):
         maxprint = self.getmaxprint()
 
-        A   = self.tocoo()
+        A = self.tocoo()
         nnz = self.getnnz()
 
         # helper function, outputs "(i,j)  v"
         def tostr(row,col,data):
-            triples = zip(zip(row,col),data)
-            return '\n'.join( [ ('  %s\t%s' % t) for t in triples] )
+            triples = zip(list(zip(row,col)),data)
+            return '\n'.join([('  %s\t%s' % t) for t in triples])
 
         if nnz > maxprint:
             half = maxprint // 2
-            out  = tostr(A.row[:half], A.col[:half], A.data[:half])
+            out = tostr(A.row[:half], A.col[:half], A.data[:half])
             out += "\n  :\t:\n"
             half = maxprint - maxprint//2
             out += tostr(A.row[-half:], A.col[-half:], A.data[-half:])
         else:
-            out  = tostr(A.row, A.col, A.data)
+            out = tostr(A.row, A.col, A.data)
 
         return out
 
-    def __nonzero__(self):  # Simple -- other ideas?
-        return self.getnnz() > 0
+    def __bool__(self):  # Simple -- other ideas?
+        if self.shape == (1, 1):
+            return True if self.nnz == 1 else False
+        else:
+            raise ValueError("The truth value of an array with more than one "
+                             "element is ambiguous. Use a.any() or a.all().")
+    __nonzero__ = __bool__
 
     # What should len(sparse) return? For consistency with dense matrices,
     # perhaps it should be the number of rows?  But for some uses the number of
@@ -208,8 +225,38 @@ class spmatrix(object):
         return self.tocsr().multiply(other)
 
     def dot(self, other):
+        """Ordinary dot product
+
+        Examples
+        --------
+        >>> import numpy as np
+        >>> from scipy.sparse import csr_matrix
+        >>> A = csr_matrix([[1, 2, 0], [0, 0, 3], [4, 0, 5]])
+        >>> v = np.array([1, 0, -1])
+        >>> A.dot(v)
+        array([ 1, -3, -1], dtype=int64)
+
+        """
         return self * other
 
+    def __eq__(self, other):
+        return self.tocsr().__eq__(other)
+
+    def __ne__(self, other):
+        return self.tocsr().__ne__(other)
+
+    def __lt__(self,other):
+        return self.tocsr().__lt__(other)
+
+    def __gt__(self,other):
+        return self.tocsr().__gt__(other)
+
+    def __le__(self,other):
+        return self.tocsr().__le__(other)
+
+    def __ge__(self,other):
+        return self.tocsr().__ge__(other)
+
     def __abs__(self):
         return abs(self.tocsr())
 
@@ -220,7 +267,7 @@ class spmatrix(object):
         return self.tocsr().__radd__(other)
 
     def __sub__(self, other):   # self - other
-        #note: this can't be replaced by self + (-other) for unsigned types
+        # note: this can't be replaced by self + (-other) for unsigned types
         return self.tocsr().__sub__(other)
 
     def __rsub__(self, other):  # other - self
@@ -237,6 +284,15 @@ class spmatrix(object):
 
         M,N = self.shape
 
+        if other.__class__ is np.ndarray:
+            # Fast path for the most common case
+            if other.shape == (N,):
+                return self._mul_vector(other)
+            elif other.shape == (N, 1):
+                return self._mul_vector(other.ravel()).reshape(M, 1)
+            elif other.ndim == 2 and other.shape[0] == N:
+                return self._mul_multivector(other)
+
         if isscalarlike(other):
             # scalar value
             return self._mul_scalar(other)
@@ -299,7 +355,7 @@ class spmatrix(object):
     def _mul_sparse_matrix(self, other):
         return self.tocsr()._mul_sparse_matrix(other)
 
-    def __rmul__(self, other): # other * self
+    def __rmul__(self, other):  # other * self
         if isscalarlike(other):
             return self.__mul__(other)
         else:
@@ -352,21 +408,21 @@ class spmatrix(object):
                 raise ValueError('exponent must be >= 0')
 
             if other == 0:
-                from construct import identity
-                return identity( self.shape[0], dtype=self.dtype )
+                from .construct import eye
+                return eye(self.shape[0], dtype=self.dtype)
             elif other == 1:
                 return self.copy()
             else:
-                result = self
-                for i in range(1,other):
-                    result = result*self
-                return result
+                tmp = self.__pow__(other//2)
+                if (other % 2):
+                    return self * tmp * tmp
+                else:
+                    return tmp * tmp
         elif isscalarlike(other):
             raise ValueError('exponent must be an integer')
         else:
             raise NotImplementedError
 
-
     def __getattr__(self, attr):
         if attr == 'A':
             return self.toarray()
@@ -402,16 +458,14 @@ class spmatrix(object):
     def _imag(self):
         return self.tocsr()._imag()
 
-
     def nonzero(self):
         """nonzero indices
 
         Returns a tuple of arrays (row,col) containing the indices
         of the non-zero elements of the matrix.
 
-        Example
-        -------
-
+        Examples
+        --------
         >>> from scipy.sparse import csr_matrix
         >>> A = csr_matrix([[1,2,0],[0,0,3],[4,0,5]])
         >>> A.nonzero()
@@ -424,7 +478,6 @@ class spmatrix(object):
         nz_mask = A.data != 0
         return (A.row[nz_mask],A.col[nz_mask])
 
-
     def getcol(self, j):
         """Returns a copy of column j of the matrix, as an (m x 1) sparse
         matrix (column vector).
@@ -432,11 +485,14 @@ class spmatrix(object):
         # Spmatrix subclasses should override this method for efficiency.
         # Post-multiply by a (n x 1) column vector 'a' containing all zeros
         # except for a_j = 1
-        from csc import csc_matrix
+        from .csc import csc_matrix
         n = self.shape[1]
-        a = csc_matrix((n, 1), dtype=self.dtype)
-        a[j, 0] = 1
-        return self * a
+        if j < 0:
+            j += n
+        if j < 0 or j >= n:
+            raise IndexError("index out of bounds")
+        col_selector = csc_matrix(([1], [[j], [0]]), shape=(n,1), dtype=self.dtype)
+        return self * col_selector
 
     def getrow(self, i):
         """Returns a copy of row i of the matrix, as a (1 x n) sparse
@@ -445,20 +501,81 @@ class spmatrix(object):
         # Spmatrix subclasses should override this method for efficiency.
         # Pre-multiply by a (1 x m) row vector 'a' containing all zeros
         # except for a_i = 1
-        from csr import csr_matrix
+        from .csr import csr_matrix
         m = self.shape[0]
-        a = csr_matrix((1, m), dtype=self.dtype)
-        a[0, i] = 1
-        return a * self
+        if i < 0:
+            i += m
+        if i < 0 or i >= m:
+            raise IndexError("index out of bounds")
+        row_selector = csr_matrix(([1], [[0], [i]]), shape=(1,m), dtype=self.dtype)
+        return row_selector * self
 
-    #def __array__(self):
+    # def __array__(self):
     #    return self.toarray()
 
-    def todense(self):
-        return np.asmatrix(self.toarray())
+    def todense(self, order=None, out=None):
+        """
+        Return a dense matrix representation of this matrix.
 
-    def toarray(self):
-        return self.tocoo().toarray()
+        Parameters
+        ----------
+        order : {'C', 'F'}, optional
+            Whether to store multi-dimensional data in C (row-major)
+            or Fortran (column-major) order in memory. The default
+            is 'None', indicating the NumPy default of C-ordered.
+            Cannot be specified in conjunction with the `out`
+            argument.
+
+        out : ndarray, 2-dimensional, optional
+            If specified, uses this array (or `numpy.matrix`) as the
+            output buffer instead of allocating a new array to
+            return. The provided array must have the same shape and
+            dtype as the sparse matrix on which you are calling the
+            method.
+
+        Returns
+        -------
+        arr : numpy.matrix, 2-dimensional
+            A NumPy matrix object with the same shape and containing
+            the same data represented by the sparse matrix, with the
+            requested memory order. If `out` was passed and was an
+            array (rather than a `numpy.matrix`), it will be filled
+            with the appropriate values and returned wrapped in a
+            `numpy.matrix` object that shares the same memory.
+        """
+        return np.asmatrix(self.toarray(order=order, out=out))
+
+    def toarray(self, order=None, out=None):
+        """
+        Return a dense ndarray representation of this matrix.
+
+        Parameters
+        ----------
+        order : {'C', 'F'}, optional
+            Whether to store multi-dimensional data in C (row-major)
+            or Fortran (column-major) order in memory. The default
+            is 'None', indicating the NumPy default of C-ordered.
+            Cannot be specified in conjunction with the `out`
+            argument.
+
+        out : ndarray, 2-dimensional, optional
+            If specified, uses this array as the output buffer
+            instead of allocating a new array to return. The provided
+            array must have the same shape and dtype as the sparse
+            matrix on which you are calling the method. For most
+            sparse types, `out` is required to be memory contiguous
+            (either C or Fortran ordered).
+
+        Returns
+        -------
+        arr : ndarray, 2-dimensional
+            An array with the same shape and containing the same
+            data represented by the sparse matrix, with the requested
+            memory order. If `out` was passed, the same object is
+            returned after being modified in-place to contain the
+            appropriate values.
+        """
+        return self.tocoo().toarray(order=order, out=out)
 
     def todok(self):
         return self.tocoo().todok()
@@ -486,15 +603,28 @@ class spmatrix(object):
         # For some sparse matrix formats more efficient methods are
         # possible -- these should override this function.
         m, n = self.shape
+
+        # Mimic numpy's casting.
+        if np.issubdtype(self.dtype, np.float_):
+            res_dtype = np.float_
+        elif (np.issubdtype(self.dtype, np.int_) or
+              np.issubdtype(self.dtype, np.bool_)):
+                res_dtype = np.int_
+        elif np.issubdtype(self.dtype, np.complex_):
+            res_dtype = np.complex_
+        else:
+            res_dtype = self.dtype
+
+        # Calculate the sum.
         if axis == 0:
             # sum over columns
-            return np.asmatrix(np.ones((1, m), dtype=self.dtype)) * self
+            return np.asmatrix(np.ones((1, m), dtype=res_dtype)) * self
         elif axis == 1:
             # sum over rows
-            return self * np.asmatrix(np.ones((n, 1), dtype=self.dtype))
+            return self * np.asmatrix(np.ones((n, 1), dtype=res_dtype))
         elif axis is None:
             # sum over rows and columns
-            return ( self * np.asmatrix(np.ones((n, 1), dtype=self.dtype)) ).sum()
+            return (self * np.asmatrix(np.ones((n, 1), dtype=res_dtype))).sum()
         else:
             raise ValueError("axis out of bounds")
 
@@ -502,12 +632,25 @@ class spmatrix(object):
         """Average the matrix over the given axis.  If the axis is None,
         average over both rows and columns, returning a scalar.
         """
+        # Mimic numpy's casting.  The int32/int64 check works around numpy
+        # 1.5.x behavior of np.issubdtype, see gh-2677.
+        if (np.issubdtype(self.dtype, np.float_) or
+            np.issubdtype(self.dtype, np.int_) or
+            self.dtype in [np.dtype('int32'), np.dtype('int64')] or
+            np.issubdtype(self.dtype, np.bool_)):
+                res_dtype = np.float_
+        elif np.issubdtype(self.dtype, np.complex_):
+            res_dtype = np.complex_
+        else:
+            res_dtype = self.dtype
+
+        # Calculate the mean.
         if axis == 0:
-            mean = self.sum(0)
+            mean = self.astype(res_dtype).sum(0)
             mean *= 1.0 / self.shape[0]
             return mean
         elif axis == 1:
-            mean = self.sum(1)
+            mean = self.astype(res_dtype).sum(1)
             mean *= 1.0 / self.shape[1]
             return mean
         elif axis is None:
@@ -518,7 +661,7 @@ class spmatrix(object):
     def diagonal(self):
         """Returns the main diagonal of the matrix
         """
-        #TODO support k != 0
+        # TODO support k != 0
         return self.tocsr().diagonal()
 
     def setdiag(self, values, k=0):
@@ -542,10 +685,21 @@ class spmatrix(object):
             for i,v in enumerate(values[:max_index]):
                 self[i, i + k] = v
 
+    def _process_toarray_args(self, order, out):
+        if out is not None:
+            if order is not None:
+                raise ValueError('order cannot be specified if out '
+                                 'is not None')
+            if out.shape != self.shape or out.dtype != self.dtype:
+                raise ValueError('out array must be same dtype and shape as '
+                                 'sparse matrix')
+            out[...] = 0.
+            return out
+        else:
+            return np.zeros(self.shape, dtype=self.dtype, order=order)
 
-from sputils import _isinstance
 
 def isspmatrix(x):
-    return _isinstance(x, spmatrix)
+    return isinstance(x, spmatrix)
 
 issparse = isspmatrix
