@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import division, print_function, absolute_import
 
 import os
 import sys
@@ -9,8 +10,8 @@ from os.path import join
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.system_info import get_info, NotFoundError
-
     from numpy.distutils.misc_util import Configuration
+    from scipy._build_utils import get_g77_abi_wrappers
 
     config = Configuration('isolve',parent_package,top_path)
 
@@ -30,17 +31,21 @@ def configuration(parent_package='',top_path=None):
                'QMRREVCOM.f.src',
 #               'SORREVCOM.f.src'
                ]
+
     Util = ['STOPTEST2.f.src','getbreak.f.src']
     sources = Util + methods + ['_iterative.pyf.src']
+    sources = [join('iterative', x) for x in sources]
+    sources += get_g77_abi_wrappers(lapack_opt)
+
     config.add_extension('_iterative',
-                         sources = [join('iterative',x) for x in sources],
-                         extra_info = lapack_opt
-                         )
+                         sources=sources,
+                         extra_info=lapack_opt)
 
     config.add_data_dir('tests')
 
     return config
 
+
 if __name__ == '__main__':
     from numpy.distutils.core import setup
 
